<?php
namespace Zeedhi\DataMining\Common\Service\DataCollector;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\DataMining\Common\Service\Mongo\Runner as MongoRunner;
use Zeedhi\DataMining\Common\Util\Environment;

class History {

    /** @var string $collectionName */
    const COLLECTION_NAME = "importHistory";

    const LOG_ERROR = "error";
    const LOG_DEBUG = "debug";

    /** @var Environment $environment */
    protected $environment;
    /** @var MongoRunner $mongoRunner */
    protected $mongoRunner;

    public function __construct(Environment $environment, MongoRunner $mongoRunner) {
        $this->environment = $environment;
        $this->mongoRunner = $mongoRunner;
    }

    public function create($process, $organizationId, $dataSourceId, $start = null, $end = null, $subprocesses = 0, $logs = array()) {
        if ($this->findByProcessId($process, false)) {
            throw new \Exception("History item duplicated. Process id {$process}", 500);
        } else {
            $historyItem = $this->factoryHistoryItem($process, $organizationId, $dataSourceId, $start, $end, $subprocesses, $logs);
            $this->mongoRunner->persist($this->getConnection(), self::COLLECTION_NAME, $historyItem);
            return $historyItem;
        }
    }

    public function update($criteria, $update, $upsert = false, $multi = false) {
        return $this->mongoRunner->update($this->getConnection(), self::COLLECTION_NAME, $criteria, $update, $upsert, $multi);
    }

    public function findByProcessId($process, $displayMessage = true) {
        $filterCriteria = new FilterCriteria(self::COLLECTION_NAME);
        $filterCriteria->addCondition('process', $process);

        $historyItem = $this->mongoRunner->findBy($this->getConnection(), $filterCriteria);

        if ($displayMessage && count($historyItem) != 1) {
            throw new \Exception("History item not found by the process id {$process}", 500);
        }

        return isset($historyItem[0]) ? $historyItem[0] : null;
    }

    public function findProcessInProgress($dataSourceId,$organizationId){
        $filterCriteria = new FilterCriteria(self::COLLECTION_NAME);
        $filterCriteria->addCondition('dataSourceId', $dataSourceId);
        $filterCriteria->addCondition('organizationId',FilterCriteria::IN,array($organizationId,intval($organizationId)));
        $conditions = array ('canceled','finalized','done', 'failed');
		$filterCriteria->addCondition('importStatus', FilterCriteria::NOT_IN, $conditions);
        $historyItem = $this->mongoRunner->findBy($this->getConnection(), $filterCriteria);
        return $historyItem;
    }

    public function setImportStatus($historyItems, $status){
        $criteria = array(
            'process' => array(
                '$in' => array_map(function($item) {
                    return $item['process'];
                }, $historyItems)
            )
        );

        $update = array(
            '$set' => array(
                'importStatus' => $status,
                'hasError' => true
            )
        );

        $this->update($criteria, $update, false, true);
    }

    public function addLog($process, $type, $message, $code = null, $trace = null) {
        $criteria = array('process' => $process);
        $update = array('$push' => array('logs' => $this->factoryLog($type, $message, $code, $trace)));
        $this->update($criteria, $update);
    }

    protected function factoryHistoryItem($process, $organizationId, $dataSourceId, $start, $end, $subprocesses, $logs, $importStatus = "start") {
        return array(
            'process'        => $process,
            'organizationId' => $organizationId,
            'dataSourceId'   => $dataSourceId,
            'start'          => $start,
            'end'            => $end ,
            'pageSize'       => null,
            'subprocesses'   => $subprocesses,
            'imported'       => 0,
            'logs'           =>  $logs,
            'hasError'       => false,
            'createDate'     => new \MongoDB\BSON\UTCDateTime(),
            'endDate'        => null,
            'elapsedTime'    => null,
            'notPage'        => null,
            'totalImported'  => 0,
            'locked'         => false,
            'importStatus'   => $importStatus
        );
    }

    public function getConnection() {
        return $this->environment->getOrganizationConnection();
    }

    public function factoryLog($type, $message, $code = null, $trace = null) {
        return array(
            'type'       => $type,
            'message'    => $message,
            'code'       => $code,
            'trace'      => $trace,
            'createDate' => new \MongoDB\BSON\UTCDateTime()
        );
    }

}