<?php
namespace Zeedhi\DataMining\Common\Service\Mongo;

use Zeedhi\Framework\DB\Mongo\Mongo as MongoConnection;
use Zeedhi\Framework\DataSource\Manager\Mongo\Query\Expr;
use Zeedhi\Framework\DataSource\Manager\Mongo\Types\Type;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\DataMining\Common\Util\Environment;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Zmart\QueryBuilders\Mongo\QueryBuilder;

class Runner {

    const MONGO_KEY_COLUMN = '_id';

    /** @var Environment $environment */
    protected $environment;
    /** @var ZmartQueryBuilder $mongoQueryBuilder */
    protected $mongoQueryBuilder;

    public function __construct(Environment $environment, QueryBuilder $mongoQueryBuilder) {
        $this->environment = $environment;
        $this->mongoQueryBuilder = $mongoQueryBuilder;
    }

    public function persist(MongoConnection $mongo, $collection, $item) {
        if (isset($item[self::MONGO_KEY_COLUMN])) {
            $documentKey = $item[self::MONGO_KEY_COLUMN];
            $criteria = $this->buildCriteriaFromRow($item);
            unset($item[self::MONGO_KEY_COLUMN]);
            $mongo->update($collection, $criteria, $item);
            $item[self::MONGO_KEY_COLUMN] = $documentKey;
        } else {
            $mongo->insert($collection, $item);
        }
        return $item;
    }

    public function update(MongoConnection $mongo, $collection, $criteria, $update, $upsert = false, $multi = false) {
        return $mongo->update($collection, $criteria, $update, $upsert, $multi);
    }

    protected function buildCriteriaFromRow($row) {
        $expression = new Expr();
        $expression->field(self::MONGO_KEY_COLUMN)->equals(Type::getType('id')->convertToDatabaseValue($row[self::MONGO_KEY_COLUMN]));
        return $expression->getQuery();
    }

    public function findBy(MongoConnection $mongo, FilterCriteria $filterCriteria) {
        return $mongo->find($filterCriteria->getDataSourceName(), $this->processFilterConditions($filterCriteria));
    }

    public function findById(MongoConnection $mongo, $collection, $id) {
        $filterCriteria = new FilterCriteria($collection);
        $filterCriteria->addCondition(self::MONGO_KEY_COLUMN, $id);
        $rows = $this->findBy($mongo, $filterCriteria);

        if (count($rows) > 0) {
            return $rows[0];
        }
        throw new \Exception("Document {$id} not found in {$collection}.", 500);
    }

    public function remove(MongoConnection $mongo, FilterCriteria $filterCriteria, $justOne = false) {
        return $mongo->remove($filterCriteria->getDataSourceName(), $this->processFilterConditions($filterCriteria), $justOne);
    }

    protected function processFilterConditions(FilterCriteria $filterCriteria) {
        $expression = new Expr();
        foreach ($filterCriteria->getConditions() as $condition) {
            $conditionValue = $this->parseToDatabaseCondition($condition);
            switch ($condition['operator']) {
                case FilterCriteria::EQ:
                    $expression->field($condition['columnName'])->equals($conditionValue);
                    break;
                case FilterCriteria::IN:
                    $expression->field($condition['columnName'])->in($conditionValue);
                    break;
                case FilterCriteria::NOT_IN:
                    $expression->field($condition['columnName'])->nin($conditionValue);
                    break;
                case FilterCriteria::LT:
                    $expression->field($condition['columnName'])->lt($conditionValue);
                    break;
                case FilterCriteria::LTE:
                    $expression->field($condition['columnName'])->lte($conditionValue);
                    break;
                case FilterCriteria::GTE:
                    $expression->field($condition['columnName'])->gte($conditionValue);
                    break;
                case FilterCriteria::GT:
                    $expression->field($condition['columnName'])->gt($conditionValue);
                    break;
                case FilterCriteria::BETWEEN:
                    list($start, $end) = $conditionValue;
                    $expression->field($condition['columnName'])->range($start, $end);
                    break;
            }
        }
        return $expression->getQuery();;
    }

    protected function parseToDatabaseCondition($condition) {
        if ($condition['columnName'] === self::MONGO_KEY_COLUMN) {
            if (is_array($condition['value'])) {
                $ids = array();
                foreach ($condition['value'] as $value)
                    $ids[] = Type::getType(Type::MONGO_ID)->convertToDatabaseValue($value);
                return $ids;
            }
            return Type::getType(Type::MONGO_ID)->convertToDatabaseValue($condition['value']);
        }
        return $condition['value'];
    }

    public function renameCollection ($currentName, $renamedName) {
        $organization = $this->environment->getOrganization();

        $mongoHost = $this->environment->getMongoHost();
        $mongoPort = $this->environment->getMongoPort();
        $dbName = $this->environment->getDbNamePrefix().$organization;
        $connectionParams = $this->environment->getOrganizationConnectionParams();
        $username = $connectionParams['username'];
        $password = $connectionParams['password'];

        $cloneCollectionCommand = "mongoexport -h {$mongoHost}:{$mongoPort} -u {$username} -p {$password} -d {$dbName} -c {$currentName}";
        $cloneCollectionCommand .= " | mongoimport -h {$mongoHost}:{$mongoPort} -u {$username} -p {$password} -d {$dbName} -c {$renamedName} --drop";

        exec($cloneCollectionCommand);
    }

    public function aggregate(MongoConnection $mongoConnection, ZmartFilterCriteria $filterCriteria, $collectionName){
        $query = $this->mongoQueryBuilder->buildQueryFromFilterCriteria($filterCriteria);

        //remove query limit
        if(isset($query[2]) && key($query[2]) === '$limit'){
            unset($query[2]);
        }

        $aggregate = $mongoConnection->aggregate($collectionName, $query);

        return $aggregate;
    }

    public function createCollection(MongoConnection $mongoConnection, $collectionName){
        return $mongoConnection->command(array(
            "create" => $collectionName
        ));
    }
}